"use strict";
/**
 * @TODO update the link once it's merged
 *
 *  Implements the spec for `Fixed Rate Swap Fees`
 *
 * - proposal - https://github.com/Sifchain/sifnode/blob/master/docs/proposals/fixed_rate_swap_fees.md
 * - tutorial - https://github.com/Sifchain/sifnode/blob/master/docs/tutorials/swap-fee-rate.md
 *
 */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.calculateSwapWithFee = exports.calculateSwapFee = exports.calculateSwap = exports.calculateSwapFeeToRowan = exports.calculateSwapToRowan = exports.calculateSwapFeeFromRowan = exports.calculateSwapFromRowan = void 0;
const bignumber_js_1 = __importDefault(require("bignumber.js"));
const ONE = (0, bignumber_js_1.default)(1);
/**
 * Calculate Swap Amount from ROWAN to EXTERNAL ASSET based on formula:
 * - (1 - f) * (1 + r) * x * Y / (x + X)
 *
 * where:
 * - f is the swap fee rate
 * - x is the input amount
 * - X is the balance of input token in the pool
 * - Y is the balance of output token in the pool
 * - r is the interest rate
 *
 * @param params {SwapParams} - swap parameters
 *
 * @returns amount obtained from swap
 *
 * @example
 *
 * const inputAmount = BigNumber(200000000000000);
 * const inputBalanceInPool = BigNumber(1999800619938006200);
 * const outputBalanceInPool = BigNumber(2000200000000000000);
 * const swapFeeRate = BigNumber(0.003);
 * const currentRatioShiftingRate = BigNumber(0);
 *
 * calculateSwapFromRowan({
 *  inputAmount,
 *  inputBalanceInPool,
 *  outputBalanceInPool,
 *  swapFeeRate,
 *  currentRatioShiftingRate
 * });
 */
function calculateSwapFromRowan({ inputAmount, inputBalanceInPool, outputBalanceInPool, swapFeeRate, currentRatioShiftingRate, }) {
    const f = (0, bignumber_js_1.default)(swapFeeRate);
    const r = (0, bignumber_js_1.default)(currentRatioShiftingRate);
    const x = (0, bignumber_js_1.default)(inputAmount);
    const X = (0, bignumber_js_1.default)(inputBalanceInPool);
    const Y = (0, bignumber_js_1.default)(outputBalanceInPool);
    // consider the formula:
    // (1 - f) * (1 + r) * x * Y / (x + X)
    const term1 = ONE.minus(f); // (1 - f)
    const term2 = ONE.plus(r); // (1 + r)
    const term3 = x.times(Y); // x * Y
    const term4 = x.plus(X); // (x + X)
    return term1.times(term2).times(term3).div(term4);
}
exports.calculateSwapFromRowan = calculateSwapFromRowan;
/**
 * Calculate Swap Fee from ROWAN to EXTERNAL ASSET based on formula:
 * - f * (1 + r) * x * Y / (x + X)
 *
 * where:
 * - f is the swap fee rate
 * - x is the input amount
 * - X is the balance of input token in the pool
 * - Y is the balance of output token in the pool
 * - r is the current ratio shifting running rate
 *
 * @param params {SwapParams} - swap parameters
 *
 * @returns swap fee amount
 *
 * @example
 *
 * const inputAmount = BigNumber(200000000000000);
 * const inputBalanceInPool = BigNumber(1999800619938006200);
 * const outputBalanceInPool = BigNumber(2000200000000000000);
 * const swapFeeRate = BigNumber(0.003);
 * const currentRatioShiftingRate = BigNumber(0);
 *
 * calculateSwapFeeFromRowan({
 *  inputAmount,
 *  inputBalanceInPool,
 *  outputBalanceInPool,
 *  swapFeeRate,
 *  currentRatioShiftingRate
 * });
 */
function calculateSwapFeeFromRowan({ inputAmount, inputBalanceInPool, outputBalanceInPool, swapFeeRate, currentRatioShiftingRate, }) {
    const f = (0, bignumber_js_1.default)(swapFeeRate);
    const r = (0, bignumber_js_1.default)(currentRatioShiftingRate);
    const x = (0, bignumber_js_1.default)(inputAmount);
    const X = (0, bignumber_js_1.default)(inputBalanceInPool);
    const Y = (0, bignumber_js_1.default)(outputBalanceInPool);
    // consider the formula:
    // f * (1 + r) * x * Y / (x + X)
    const term1 = f.times(ONE.plus(r)); // f * (1 + r)
    const term2 = x.times(Y); // x * Y
    const term3 = x.plus(X); // (x + X)
    return term1.times(term2).div(term3);
}
exports.calculateSwapFeeFromRowan = calculateSwapFeeFromRowan;
/**
 * Calculate Swap Amount from EXTERNAL ASSET to ROWAN based on formula:
 * - (1 - f) * x * Y / ((x + X)(1 + r))
 *
 * where:
 * - f is the swap fee rate
 * - x is the input amount
 * - X is the balance of input token in the pool
 * - Y is the balance of output token in the pool
 * - r is the current ratio shifting running rate
 *
 * @param params {SwapParams} - swap parameters
 *
 * @returns amount obtained from swap
 *
 * @example
 *
 * const inputAmount = BigNumber(200000000000000);
 * const inputBalanceInPool = BigNumber(1999800619938006200);
 * const outputBalanceInPool = BigNumber(2000200000000000000);
 * const swapFeeRate = BigNumber(0.003);
 * const currentRatioShiftingRate = BigNumber(0);
 *
 * calculateSwapToRowan({
 *  inputAmount,
 *  inputBalanceInPool,
 *  outputBalanceInPool,
 *  swapFeeRate,
 *  currentRatioShiftingRate
 * });
 */
function calculateSwapToRowan({ inputAmount, inputBalanceInPool, outputBalanceInPool, swapFeeRate, currentRatioShiftingRate, }) {
    const f = (0, bignumber_js_1.default)(swapFeeRate);
    const r = (0, bignumber_js_1.default)(currentRatioShiftingRate);
    const x = (0, bignumber_js_1.default)(inputAmount);
    const X = (0, bignumber_js_1.default)(inputBalanceInPool);
    const Y = (0, bignumber_js_1.default)(outputBalanceInPool);
    // consider the formula:
    // (1 - f) * x * Y / ((x + X)(1 + r))
    const term1 = ONE.minus(f); // (1 - f)
    const term2 = x.times(Y); // x * Y
    const term3 = x.plus(X); // (x + X)
    const term4 = ONE.plus(r); // (1 + r)
    return term1.times(term2).div(term3.times(term4));
}
exports.calculateSwapToRowan = calculateSwapToRowan;
/**
 * Calculate Swap Fee from EXTERNAL ASSET to ROWAN based on formula:
 * - f * x * Y / ((x + X)(1 + r))
 *
 * where:
 * - f is the swap fee rate
 * - x is the input amount
 * - X is the balance of input token in the pool
 * - Y is the balance of output token in the pool
 * - r is the current ratio shifting running rate
 *
 * @param params {SwapParams} - swap parameters
 *
 * @returns swap fee amount
 *
 * @example
 *
 * const inputAmount = BigNumber(200000000000000);
 * const inputBalanceInPool = BigNumber(1999800619938006200);
 * const outputBalanceInPool = BigNumber(2000200000000000000);
 * const swapFeeRate = BigNumber(0.003);
 * const currentRatioShiftingRate = BigNumber(0);
 *
 * calculateSwapFeeToRowan({
 *  inputAmount,
 *  inputBalanceInPool,
 *  outputBalanceInPool,
 *  swapFeeRate,
 *  currentRatioShiftingRate
 * });
 */
function calculateSwapFeeToRowan({ inputAmount, inputBalanceInPool, outputBalanceInPool, swapFeeRate, currentRatioShiftingRate, }) {
    const f = (0, bignumber_js_1.default)(swapFeeRate);
    const r = (0, bignumber_js_1.default)(currentRatioShiftingRate);
    const x = (0, bignumber_js_1.default)(inputAmount);
    const X = (0, bignumber_js_1.default)(inputBalanceInPool);
    const Y = (0, bignumber_js_1.default)(outputBalanceInPool);
    // consider the formula:
    // f * x * Y / ((x + X)(1 + r))
    const term1 = f.times(x).times(Y); // f * x * Y
    const term2 = x.plus(X); // (x + X)
    const term3 = ONE.plus(r); // (1 + r)
    return term1.div(term2.times(term3));
}
exports.calculateSwapFeeToRowan = calculateSwapFeeToRowan;
const calculateSwap = (params, toRowan) => {
    const fn = toRowan ? calculateSwapToRowan : calculateSwapFromRowan;
    return fn(params);
};
exports.calculateSwap = calculateSwap;
const calculateSwapFee = (params, toRowan) => {
    const fn = toRowan ? calculateSwapFeeToRowan : calculateSwapFeeFromRowan;
    return fn(params);
};
exports.calculateSwapFee = calculateSwapFee;
const calculateSwapWithFee = (params, toRowan) => ({
    swap: (0, exports.calculateSwap)(params, toRowan),
    fee: (0, exports.calculateSwapFee)(params, toRowan),
});
exports.calculateSwapWithFee = calculateSwapWithFee;
