"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.calculatePriceImpact = exports.calculateLiquidityProviderFee = exports.calculateSwapAmountNeeded = exports.calculateSwapResult = void 0;
const bignumber_js_1 = __importDefault(require("bignumber.js"));
/**
 * calculate Swap Result based on formula ( x * X * Y ) / ( x + X ) ^ 2
 * @param fromAmount
 * @param fromCoinPoolAmount
 * @param toCoinPoolAmount
 * @returns amount obtained from swap
 */
const _calculateSwapResult = (fromAmount, fromCoinPoolAmount, toCoinPoolAmount) => {
    const f = new bignumber_js_1.default(fromAmount);
    const fp = new bignumber_js_1.default(fromCoinPoolAmount);
    const tp = new bignumber_js_1.default(toCoinPoolAmount);
    if (f.isZero() || fp.isZero() || tp.isZero()) {
        return new bignumber_js_1.default(0);
    }
    const xPlusX = f.plus(fp);
    return f.times(fp).times(tp).div(xPlusX.times(xPlusX));
};
/**
 * calculate Swap Result based on formula (( x * X * Y ) / ( x + X ) ^ 2) * (1 + adjustment / 100)
 * @param fromAmount
 * @param fromCoinPoolAmount
 * @param toCoinPoolAmount
 * @param adjustment PMTP purchasing power adjustment, `undefined` if no adjustment is needed
 * @returns amount obtained from swap
 */
const calculateSwapResult = (fromAmount, fromCoinPoolAmount, toCoinPoolAmount, adjustment) => {
    const f = new bignumber_js_1.default(fromAmount);
    const fp = new bignumber_js_1.default(fromCoinPoolAmount);
    const tp = new bignumber_js_1.default(toCoinPoolAmount);
    if (f.isZero() || fp.isZero() || tp.isZero()) {
        return new bignumber_js_1.default(0);
    }
    if (adjustment === undefined || new bignumber_js_1.default(adjustment).isZero()) {
        return _calculateSwapResult(f, fp, tp);
    }
    const a = new bignumber_js_1.default(adjustment);
    const adjustmentPercentage = a.div(100000000000000000000);
    return _calculateSwapResult(f, fp, tp).times(adjustmentPercentage.plus(1));
};
exports.calculateSwapResult = calculateSwapResult;
/**
 * formula: S = (x * X * Y) / (x + X) ^ 2
 * reverse Formula: x = ( -2*X*S + X*Y - X*sqrt( Y*(Y - 4*S) ) ) / 2*S
 * ok to accept a little precision loss as reverse swap amount can be rough
 * @param targetAmount
 * @param targetCoinPoolAmount
 * @param fromCoinPoolAmount
 * @returns
 */
const calculateSwapAmountNeeded = (targetAmount, targetCoinPoolAmount, fromCoinPoolAmount) => {
    const t = new bignumber_js_1.default(targetAmount);
    const tp = new bignumber_js_1.default(targetCoinPoolAmount);
    const fp = new bignumber_js_1.default(fromCoinPoolAmount);
    // Adding a check here because sqrt of a negative number will throw an exception
    if (t.isZero() || tp.isZero() || t.times(4).gt(fp)) {
        return new bignumber_js_1.default(0);
    }
    const term1 = new bignumber_js_1.default(-2).times(tp).times(t);
    const term2 = tp.times(fp);
    const underRoot = fp.times(fp.minus(t.times(4)));
    const term3 = tp.times(underRoot.sqrt());
    const numerator = term1.plus(term2).minus(term3);
    const denominator = t.times(2);
    const x = numerator.div(denominator);
    return x.gte(0) ? x : new bignumber_js_1.default(0);
};
exports.calculateSwapAmountNeeded = calculateSwapAmountNeeded;
/**
 * calculate Provider Fee according to the formula: ( x^2 * Y ) / ( x + X )^2
 * @param fromAmount swap Amount
 * @param fromCoinPoolAmount external Balance
 * @param toCoinPoolAmount native Balance
 * @returns providerFee
 */
const calculateLiquidityProviderFee = (fromAmount, fromCoinPoolAmount, toCoinPoolAmount) => {
    const f = new bignumber_js_1.default(fromAmount);
    const fp = new bignumber_js_1.default(fromCoinPoolAmount);
    const tp = new bignumber_js_1.default(toCoinPoolAmount);
    if (f.isZero() || fp.isZero() || tp.isZero()) {
        return new bignumber_js_1.default(0);
    }
    const xPlusX = f.plus(fp);
    return f.times(f).times(tp).div(xPlusX.times(xPlusX));
};
exports.calculateLiquidityProviderFee = calculateLiquidityProviderFee;
/**
 * calculate price impact according to the formula (x) / (x + X)
 * @param fromAmount swap Amount
 * @param fromCoinPoolAmount external Balance
 * @returns
 */
const calculatePriceImpact = (fromAmount, fromCoinPoolAmount) => {
    const f = new bignumber_js_1.default(fromAmount);
    if (f.isZero()) {
        return new bignumber_js_1.default(0);
    }
    const denominator = f.plus(fromCoinPoolAmount);
    return f.div(denominator);
};
exports.calculatePriceImpact = calculatePriceImpact;
