/**
 * @TODO update the link once it's merged
 *
 *  Implements the spec for `Fixed Rate Swap Fees`
 *
 * - proposal - https://github.com/Sifchain/sifnode/blob/master/docs/proposals/fixed_rate_swap_fees.md
 * - tutorial - https://github.com/Sifchain/sifnode/blob/master/docs/tutorials/swap-fee-rate.md
 *
 */
import BigNumber from "bignumber.js";
export declare type SwapParams = {
    /**
     * amount to be swapped from
     */
    inputAmount: BigNumber.Value;
    /**
     * amount of input asset in the pool
     */
    inputBalanceInPool: BigNumber.Value;
    /**
     * amount of output asset in the pool
     */
    outputBalanceInPool: BigNumber.Value;
    /**
     * current swap fee rate (sifnode gov param)
     */
    swapFeeRate: BigNumber.Value;
    /**
     * current PMTP ratio shifting rate
     */
    currentRatioShiftingRate: BigNumber.Value;
};
/**
 * Calculate Swap Amount from ROWAN to EXTERNAL ASSET based on formula:
 * - (1 - f) * (1 + r) * x * Y / (x + X)
 *
 * where:
 * - f is the swap fee rate
 * - x is the input amount
 * - X is the balance of input token in the pool
 * - Y is the balance of output token in the pool
 * - r is the interest rate
 *
 * @param params {SwapParams} - swap parameters
 *
 * @returns amount obtained from swap
 *
 * @example
 *
 * const inputAmount = BigNumber(200000000000000);
 * const inputBalanceInPool = BigNumber(1999800619938006200);
 * const outputBalanceInPool = BigNumber(2000200000000000000);
 * const swapFeeRate = BigNumber(0.003);
 * const currentRatioShiftingRate = BigNumber(0);
 *
 * calculateSwapFromRowan({
 *  inputAmount,
 *  inputBalanceInPool,
 *  outputBalanceInPool,
 *  swapFeeRate,
 *  currentRatioShiftingRate
 * });
 */
export declare function calculateSwapFromRowan({ inputAmount, inputBalanceInPool, outputBalanceInPool, swapFeeRate, currentRatioShiftingRate, }: SwapParams): BigNumber;
/**
 * Calculate Swap Fee from ROWAN to EXTERNAL ASSET based on formula:
 * - f * (1 + r) * x * Y / (x + X)
 *
 * where:
 * - f is the swap fee rate
 * - x is the input amount
 * - X is the balance of input token in the pool
 * - Y is the balance of output token in the pool
 * - r is the current ratio shifting running rate
 *
 * @param params {SwapParams} - swap parameters
 *
 * @returns swap fee amount
 *
 * @example
 *
 * const inputAmount = BigNumber(200000000000000);
 * const inputBalanceInPool = BigNumber(1999800619938006200);
 * const outputBalanceInPool = BigNumber(2000200000000000000);
 * const swapFeeRate = BigNumber(0.003);
 * const currentRatioShiftingRate = BigNumber(0);
 *
 * calculateSwapFeeFromRowan({
 *  inputAmount,
 *  inputBalanceInPool,
 *  outputBalanceInPool,
 *  swapFeeRate,
 *  currentRatioShiftingRate
 * });
 */
export declare function calculateSwapFeeFromRowan({ inputAmount, inputBalanceInPool, outputBalanceInPool, swapFeeRate, currentRatioShiftingRate, }: SwapParams): BigNumber;
/**
 * Calculate Swap Amount from EXTERNAL ASSET to ROWAN based on formula:
 * - (1 - f) * x * Y / ((x + X)(1 + r))
 *
 * where:
 * - f is the swap fee rate
 * - x is the input amount
 * - X is the balance of input token in the pool
 * - Y is the balance of output token in the pool
 * - r is the current ratio shifting running rate
 *
 * @param params {SwapParams} - swap parameters
 *
 * @returns amount obtained from swap
 *
 * @example
 *
 * const inputAmount = BigNumber(200000000000000);
 * const inputBalanceInPool = BigNumber(1999800619938006200);
 * const outputBalanceInPool = BigNumber(2000200000000000000);
 * const swapFeeRate = BigNumber(0.003);
 * const currentRatioShiftingRate = BigNumber(0);
 *
 * calculateSwapToRowan({
 *  inputAmount,
 *  inputBalanceInPool,
 *  outputBalanceInPool,
 *  swapFeeRate,
 *  currentRatioShiftingRate
 * });
 */
export declare function calculateSwapToRowan({ inputAmount, inputBalanceInPool, outputBalanceInPool, swapFeeRate, currentRatioShiftingRate, }: SwapParams): BigNumber;
/**
 * Calculate Swap Fee from EXTERNAL ASSET to ROWAN based on formula:
 * - f * x * Y / ((x + X)(1 + r))
 *
 * where:
 * - f is the swap fee rate
 * - x is the input amount
 * - X is the balance of input token in the pool
 * - Y is the balance of output token in the pool
 * - r is the current ratio shifting running rate
 *
 * @param params {SwapParams} - swap parameters
 *
 * @returns swap fee amount
 *
 * @example
 *
 * const inputAmount = BigNumber(200000000000000);
 * const inputBalanceInPool = BigNumber(1999800619938006200);
 * const outputBalanceInPool = BigNumber(2000200000000000000);
 * const swapFeeRate = BigNumber(0.003);
 * const currentRatioShiftingRate = BigNumber(0);
 *
 * calculateSwapFeeToRowan({
 *  inputAmount,
 *  inputBalanceInPool,
 *  outputBalanceInPool,
 *  swapFeeRate,
 *  currentRatioShiftingRate
 * });
 */
export declare function calculateSwapFeeToRowan({ inputAmount, inputBalanceInPool, outputBalanceInPool, swapFeeRate, currentRatioShiftingRate, }: SwapParams): BigNumber;
export declare const calculateSwap: (params: SwapParams, toRowan: boolean) => BigNumber;
export declare const calculateSwapFee: (params: SwapParams, toRowan: boolean) => BigNumber;
export declare const calculateSwapWithFee: (params: SwapParams, toRowan: boolean) => {
    swap: BigNumber;
    fee: BigNumber;
};
